﻿using System;
using System.IO;
using System.Threading;

namespace TotpAuth {
    class Program {
        private const string FILE_NAME = "secret.txt";

        static void Main( string[] args ) {
            // show error and prompt user when secret file does not exist
            if ( !File.Exists( FILE_NAME ) ) {
                Console.WriteLine( "File `" + FILE_NAME + "` does not exist." );
                Console.WriteLine( "Put your secret in file `" + FILE_NAME + "` and restart the program." );
                Console.WriteLine( "Press ENTER to exit." );
                Console.ReadLine();
                return;
            }
            
            // file exists, so we can start
            Console.WriteLine( "Reading secret from file `" + FILE_NAME + "`" );
            Console.WriteLine( "Generating an authorization code every " + Totp.MaxLifetime );
            Console.WriteLine( "PRESS ANY KEY TO EXIT\n" );

            string base32Secret = File.ReadAllText( FILE_NAME );
            while ( !Console.KeyAvailable ) {
                Totp totp = new Totp( base32Secret );
                Console.WriteLine( totp );

                // busy wait until either the TOTP expires, or the user exits
                while ( !Console.KeyAvailable && !totp.IsExpired ) {
                    Thread.Sleep( TimeSpan.FromMilliseconds( 100 ) );
                }
            }
        }
    }
}
